/**
 * This file is part of IoTool Arduino Library.
 * 
 * IoTool Arduino Library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * IoTool Arduino Library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with IoTool Arduino Library. If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * IoTool Arduino Library demo
 *
 * This sketch creates three IoTool sensors which take data from 101 acceleration
 * sensor. Before sensors will be visible in the service sync is required. Navigate
 * to service preferences on your Android device and click "Sync". Make sure that
 * correct MAC is set and "Use Bluetooth Low Energy" checkbox is checked, before
 * attemping to sync.
 */

#include <IoTool.h>
#include <CurieIMU.h>

// Create IoTool object
IoTool ioTool;
// Create sensor for each accelerometer axis, with 2 decimal points of precision
IoToolSensor accX("accX", "Acceleration X-axis", "Acc.X", "g", 2);
IoToolSensor accY("accY", "Acceleration Y-axis", "Acc.Y", "g", 2);
IoToolSensor accZ("accZ", "Acceleration Z-axis", "Acc.Z", "g", 2);

// This sketch will be using 101 internal Bluetooth Low Energy, in order
// to use Bluetooth Low energy with Android service CurieBLESerial object
// is needed. This object emulates serial connection over Bluetooth
// Low energy, which enables use of same objects and methods as in
// Bluetooth Classic version of the sketch.
CurieBLESerial cSerial;

// Time keeping variable - used to keep track of last sensor write time.
unsigned long previousNow;

void setup() {
     // Start 101 IMU to enable us to receive data from the accelerometer
     CurieIMU.begin();
     // Set accelerometer range to +-2g
     CurieIMU.setAccelerometerRange(2);

     // Initialize 101 Bluetooth Low Energy serial emulation
     cSerial.begin();
     // Attach Bluetooth Low Energy serial object to the IoTool object
     ioTool.setBLESerial(cSerial);

     // Add sensors to the IoTool object
     ioTool.addSensor(accX); 
     ioTool.addSensor(accY); 
     ioTool.addSensor(accZ); 
}

void loop() {
  // Execute main loop of the IoTool object
  ioTool.loop();

  // DO NOT use delay function in loop, as it will interfere with IoTool's
  // loop, resulting in unexecuted actions. Use millis() or Timer library
  // (http://playground.arduino.cc/Code/Timer) instead.
  // Check if 200ms has already passed since last sensor write
  if (millis() - previousNow >= 200) {
    // Keep track of last time when this procedure was executed
    previousNow = millis();

    // Initialize variables for raw sensor data
    int xRaw, yRaw, zRaw;
    // Initialize variables for converted sensor data
    float x, y, z;

    // Read current accelerometer values in to the raw sensor data variables
    CurieIMU.readAccelerometer(xRaw, yRaw, zRaw);
    // Convert raw sensor value of X-axis to g values
    x = convertRawAcceleration(xRaw);
    // Write converted value to IoTool X-axis sensor
    accX.write(x);
    // Convert raw sensor value of Y-axis to g values	
    y = convertRawAcceleration(yRaw);
    // Write converted value to IoTool Y-axis sensor
    accY.write(y);
    // Convert raw sensor value of Z-axis to g values	
    z = convertRawAcceleration(zRaw);
    // Write converted value to IoTool Z-axis sensor	
    accZ.write(z);
  }
}

// Function to convert raw sensor data to g values
float convertRawAcceleration(int aRaw) {
  // since we are using 2G range
  // -2g maps to a raw value of -32768
  // +2g maps to a raw value of 32767
  
  return (aRaw * 2.0) / 32768.0;
}
